# Generated by Django 3.2.13 on 2022-08-26 01:06

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("edc_pharmacy", "0010_auto_20220708_0813"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalrx",
            name="action_identifier",
            field=models.CharField(blank=True, db_index=True, max_length=50, null=True),
        ),
        migrations.AlterField(
            model_name="rx",
            name="action_identifier",
            field=models.CharField(blank=True, max_length=50, null=True, unique=True),
        ),
    ]
